MODULE iceadv
   !!======================================================================
   !!                       ***  MODULE iceadv   ***
   !! LIM transport ice model : sea-ice advection/diffusion
   !!======================================================================
   !! History : LIM-2 ! 2000-01 (M.A. Morales Maqueda, H. Goosse, and T. Fichefet)  Original code
   !!            3.0  ! 2005-11 (M. Vancoppenolle)   Multi-layer sea ice, salinity variations
   !!            4.0  ! 2011-02 (G. Madec) dynamical allocation
   !!----------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3'                                       LIM3 sea-ice model
   !!----------------------------------------------------------------------
   !!   ice_adv       : advection/diffusion process of sea ice
   !!----------------------------------------------------------------------
   USE phycst         ! physical constant
   USE dom_oce        ! ocean domain
   USE sbc_oce , ONLY : nn_fsbc   ! frequency of sea-ice call
   USE ice            ! sea-ice: variables
   USE icevar         ! sea-ice: operations
   USE iceadv_prather ! sea-ice: advection scheme (Prather)
   USE iceadv_umx     ! sea-ice: advection scheme (ultimate-macho)
   USE icectl         ! sea-ice: control prints
   !
   USE in_out_manager ! I/O manager
   USE lbclnk         ! lateral boundary conditions -- MPP exchanges
   USE lib_mpp        ! MPP library
   USE prtctl         ! Print control
   USE lib_fortran    ! Fortran utilities (allows no signed zero when 'key_nosignedzero' defined)  
   USE timing         ! Timing
   USE iom            !

   IMPLICIT NONE
   PRIVATE

   PUBLIC   ice_adv        ! called by icestp
   PUBLIC   ice_adv_init   ! called by icestp

   INTEGER, PUBLIC ::              nice_dyn   ! choice of the type of advection scheme
   !                                  ! associated indices:
   INTEGER, PUBLIC, PARAMETER ::   np_dynNO   = 0   ! no ice dynamics and ice advection
   INTEGER, PUBLIC, PARAMETER ::   np_dynFULL = 1   ! full ice dynamics  (rheology + advection + ridging/rafting + correction)
   INTEGER, PUBLIC, PARAMETER ::   np_dyn     = 2   ! no ridging/rafting (rheology + advection                   + correction)
   INTEGER, PUBLIC, PARAMETER ::   np_dynPURE = 3   ! pure dynamics      (rheology + advection) 

   !! * Substitution
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/ICE 4.0 , NEMO Consortium (2017)
   !! $Id: iceadv.F90 8373 2017-07-25 17:44:54Z clem $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE ice_adv( kt ) 
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE ice_adv ***
      !!                    
      !! ** purpose : advection of sea ice
      !!
      !! ** method  : variables included in the process are scalar,   
      !!     other values are considered as second order. 
      !!     For advection, one can choose between
      !!     a) an Ultimate-Macho scheme (whose order is defined by nn_UMx) => ln_adv_UMx
      !!     b) and a second order Prather scheme => ln_adv_Pra
      !!
      !! ** action :
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt   ! number of iteration
      !
      INTEGER  ::   ji, jj, jk, jl, jt      ! dummy loop indices
      INTEGER  ::   initad                  ! number of sub-timestep for the advection
      REAL(wp) ::   zcfl , zusnit           !   -      -
      CHARACTER(len=80) :: cltmp
      !
      REAL(wp) ::    zvi_b, zsmv_b, zei_b, zfs_b, zfw_b, zft_b
      REAL(wp) ::    zdv
      REAL(wp), DIMENSION(jpi,jpj)           ::   zatold, zeiold, zesold, zsmvold 
      REAL(wp), DIMENSION(jpi,jpj,jpl)       ::   zhimax, zviold, zvsold
      !!---------------------------------------------------------------------
      IF( nn_timing == 1 )  CALL timing_start('iceadv')
 
      IF( kt == nit000 .AND. lwp ) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'ice_adv: sea-ice advection'
         WRITE(numout,*) '~~~~~~~'
      ENDIF
      
      CALL ice_var_agg( 1 ) ! integrated values + ato_i

      ! conservation test
      IF( ln_limdiachk )   CALL ice_cons_hsm(0, 'iceadv', zvi_b, zsmv_b, zei_b, zfw_b, zfs_b, zft_b)
      
      ! store old values for diag
      zviold (:,:,:) = v_i(:,:,:)
      zvsold (:,:,:) = v_s(:,:,:)
      zsmvold(:,:)   = SUM( smv_i(:,:,:), dim=3 )
      zeiold (:,:)   = et_i(:,:)
      zesold (:,:)   = et_s(:,:)

      ! Thickness correction init.
      zatold(:,:) = at_i
      WHERE( a_i(:,:,:) >= epsi20 )
         ht_i(:,:,:) = v_i(:,:,:) / a_i(:,:,:)
         ht_s(:,:,:) = v_s(:,:,:) / a_i(:,:,:)
      ELSEWHERE
         ht_i(:,:,:) = 0._wp
         ht_s(:,:,:) = 0._wp         
      END WHERE
         
      ! Record max of the surrounding ice thicknesses for correction in case advection creates ice too thick
      zhimax(:,:,:) = ht_i(:,:,:) + ht_s(:,:,:)
      DO jl = 1, jpl
         DO jj = 2, jpjm1
            DO ji = 2, jpim1
!!gm use of MAXVAL here is very probably less efficient than expending the 9 values
               zhimax(ji,jj,jl) = MAX( epsi20, MAXVAL( ht_i(ji-1:ji+1,jj-1:jj+1,jl) + ht_s(ji-1:ji+1,jj-1:jj+1,jl) ) )
            END DO
         END DO
      END DO
      CALL lbc_lnk( zhimax(:,:,:), 'T', 1. )
      
      !----------
      ! Advection
      !----------
      IF( ln_adv_UMx ) THEN                    !-- ULTIMATE-MACHO scheme
         CALL ice_adv_umx( kt, u_ice, v_ice,  &
            &              ato_i, v_i, v_s, smv_i, oa_i, a_i, a_ip, v_ip, e_s, e_i )
         
      ELSEIF( ln_adv_Pra ) THEN                !-- PRATHER scheme
         CALL ice_adv_prather( kt, u_ice, v_ice,  &
            &                  ato_i, v_i, v_s, smv_i, oa_i, a_i, a_ip, v_ip, e_s, e_i )
         
      ENDIF

      ! total ice fraction
      at_i(:,:) = a_i(:,:,1)
      DO jl = 2, jpl
         at_i(:,:) = at_i(:,:) + a_i(:,:,jl)
      END DO
      
      !------------
      ! diagnostics
      !------------
      DO jj = 1, jpj
         DO ji = 1, jpi
            diag_trp_ei (ji,jj) = ( SUM( e_i  (ji,jj,1:nlay_i,:) ) -  zeiold(ji,jj) ) * r1_rdtice
            diag_trp_es (ji,jj) = ( SUM( e_s  (ji,jj,1:nlay_s,:) ) -  zesold(ji,jj) ) * r1_rdtice
            diag_trp_smv(ji,jj) = ( SUM( smv_i(ji,jj,:)          ) - zsmvold(ji,jj) ) * r1_rdtice
            diag_trp_vi (ji,jj) =   SUM(   v_i(ji,jj,:)            -  zviold(ji,jj,:) ) * r1_rdtice
            diag_trp_vs (ji,jj) =   SUM(   v_s(ji,jj,:)            -  zvsold(ji,jj,:) ) * r1_rdtice
         END DO
      END DO
      IF( iom_use('icetrp') )   CALL iom_put( "icetrp" , diag_trp_vi * rday  )         ! ice volume transport
      IF( iom_use('snwtrp') )   CALL iom_put( "snwtrp" , diag_trp_vs * rday  )         ! snw volume transport
      IF( iom_use('saltrp') )   CALL iom_put( "saltrp" , diag_trp_smv * rday * rhoic ) ! salt content transport
      IF( iom_use('deitrp') )   CALL iom_put( "deitrp" , diag_trp_ei         )         ! advected ice enthalpy (W/m2)
      IF( iom_use('destrp') )   CALL iom_put( "destrp" , diag_trp_es         )         ! advected snw enthalpy (W/m2)
      
      !--------------------------------------
      ! Thickness correction in case too high
      !--------------------------------------
      IF( nn_icedyn == 2 ) THEN
         !
         CALL ice_var_zapsmall                       !-- zap small areas
         !
         DO jl = 1, jpl
            DO jj = 1, jpj
               DO ji = 1, jpi
                  IF ( v_i(ji,jj,jl) > 0._wp ) THEN  !-- bound to zhimax
                     !
                     ht_i  (ji,jj,jl) = v_i (ji,jj,jl) / a_i(ji,jj,jl)
                     ht_s  (ji,jj,jl) = v_s (ji,jj,jl) / a_i(ji,jj,jl)
                     zdv = v_i(ji,jj,jl) + v_s(ji,jj,jl) - zviold(ji,jj,jl) - zvsold(ji,jj,jl)  
                     !
                     IF ( ( zdv >  0.0 .AND. (ht_i(ji,jj,jl)+ht_s(ji,jj,jl)) > zhimax(ji,jj,jl) .AND. zatold(ji,jj) < 0.80 ) .OR. &
                        & ( zdv <= 0.0 .AND. (ht_i(ji,jj,jl)+ht_s(ji,jj,jl)) > zhimax(ji,jj,jl) ) ) THEN
                        a_i (ji,jj,jl) = ( v_i(ji,jj,jl) + v_s(ji,jj,jl) ) / zhimax(ji,jj,jl)
                        ht_i(ji,jj,jl) =   v_i(ji,jj,jl) / a_i(ji,jj,jl)
                     ENDIF
                     !
                  ENDIF
               END DO
            END DO
         END DO
                  
         WHERE( ht_i(:,:,jpl) > hi_max(jpl) )        !-- bound ht_i to hi_max (99 m)
            ht_i(:,:,jpl) = hi_max(jpl)
            a_i (:,:,jpl) = v_i(:,:,jpl) / hi_max(jpl)
         END WHERE

         IF ( nn_pnd_scheme > 0 ) THEN               !-- correct pond fraction to avoid a_ip > a_i
            WHERE( a_ip(:,:,:) > a_i(:,:,:) )   a_ip(:,:,:) = a_i(:,:,:)
         ENDIF
         !
      ENDIF
         
      !------------------------------------------------------------
      ! Impose a_i < amax if no ridging/rafting or in mono-category
      !------------------------------------------------------------
      IF( l_piling ) THEN                            !-- simple conservative piling, comparable with 1-cat models
         at_i(:,:) = SUM( a_i(:,:,:), dim=3 )
         DO jl = 1, jpl
            WHERE( at_i(:,:) > epsi20 )
               a_i(:,:,jl) = a_i(:,:,jl) * (  1._wp + MIN( rn_amax_2d(:,:) - at_i(:,:) , 0._wp ) / at_i(:,:)  )
            END WHERE
         END DO
      ENDIF
      
      ! agglomerate variables 
      vt_i(:,:) = SUM( v_i(:,:,:), dim=3 )
      vt_s(:,:) = SUM( v_s(:,:,:), dim=3 )
      at_i(:,:) = SUM( a_i(:,:,:), dim=3 )

      ! MV MP 2016 (remove once we get rid of a_i_frac and ht_i)
      IF ( nn_pnd_scheme > 0 ) THEN
          at_ip(:,:) = SUM( a_ip(:,:,:), dim = 3 )
          vt_ip(:,:) = SUM( v_ip(:,:,:), dim = 3 )
      ENDIF
      ! END MP 2016
      
      ! open water = 1 if at_i=0 
      WHERE( at_i == 0._wp ) ato_i = 1._wp 
      
      ! conservation test
      IF( ln_limdiachk )   CALL ice_cons_hsm(1, 'iceadv', zvi_b, zsmv_b, zei_b, zfw_b, zfs_b, zft_b)
        
      ! --------------
      ! control prints
      ! --------------
      IF( ln_limctl )   CALL ice_prt( kt, iiceprt, jiceprt,-1, ' - ice dyn & trp - ' )
      !
      IF( nn_timing == 1 )  CALL timing_stop('iceadv')
      !
   END SUBROUTINE ice_adv


   SUBROUTINE ice_adv_init
      !!-------------------------------------------------------------------
      !!                  ***  ROUTINE ice_adv_init  ***
      !!
      !! ** Purpose : Physical constants and parameters linked to the ice
      !!      dynamics
      !!
      !! ** Method  :  Read the namice_adv namelist and check the ice-dynamic
      !!       parameter values called at the first timestep (nit000)
      !!
      !! ** input   :   Namelist namice_adv
      !!-------------------------------------------------------------------
      INTEGER ::   ios   ! Local integer output status for namelist read
      !!
      NAMELIST/namice_adv/ ln_icedyn, nn_icedyn, rn_uice, rn_vice, ln_adv_Pra, ln_adv_UMx, nn_UMx
      !!-------------------------------------------------------------------
      !
      REWIND( numnam_ice_ref )         ! Namelist namice_adv in reference namelist : Ice dynamics
      READ  ( numnam_ice_ref, namice_adv, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namice_adv in reference namelist', lwp )
      !
      REWIND( numnam_ice_cfg )         ! Namelist namice_adv in configuration namelist : Ice dynamics
      READ  ( numnam_ice_cfg, namice_adv, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namice_adv in configuration namelist', lwp )
      IF(lwm) WRITE ( numoni, namice_adv )
      !
      IF(lwp) THEN                     ! control print
         WRITE(numout,*)
         WRITE(numout,*) 'ice_adv_init : ice parameters for ice dynamics '
         WRITE(numout,*) '~~~~~~~~~~~~'
         WRITE(numout,*) '   Namelist namice_adv'
         WRITE(numout,*) '      Ice dynamics       (T) or not (F)                         ln_icedyn  = ', ln_icedyn
         WRITE(numout,*) '         associated switch                                      nn_icedyn  = ', nn_icedyn
         WRITE(numout,*) '            =2 all processes (default option)'
         WRITE(numout,*) '            =1 advection only (no ridging/rafting)'
         WRITE(numout,*) '            =0 advection only with prescribed velocity given by '
         WRITE(numout,*) '               a uniform field               (u,v)_ice = (rn_uice,rn_vice) = (', rn_uice,',', rn_vice,')'
         WRITE(numout,*) '      advection scheme for ice transport (limtrp)'
         WRITE(numout,*) '         type of advection scheme (Prather)                     ln_adv_Pra = ', ln_adv_Pra 
         WRITE(numout,*) '         type of advection scheme (Ulimate-Macho)               ln_adv_UMx = ', ln_adv_UMx 
         WRITE(numout,*) '            order of the Ultimate-Macho scheme                      nn_UMx = ', nn_UMx
      ENDIF
      !
      !                             ! set the choice of ice dynamics
      IF( lk_c1d .OR. .NOT. ln_icedyn ) THEN
         nice_dyn = np_dynNO                    !--- no dynamics
      ELSE
         SELECT CASE( nn_icedyn )
         CASE( 2 )                    
            IF( nn_monocat /= 2 ) THEN          !--- full dynamics (rheology + advection + ridging/rafting + correction)
               nice_dyn = np_dynFULL
            ELSE
               nice_dyn = np_dyn                !--- dynamics without ridging/rafting
            ENDIF
         CASE( 0 , 1 )                          !--- dynamics without ridging/rafting and correction 
            nice_dyn = np_dynPURE
         END SELECT
      ENDIF
      !                                         !--- simple conservative piling, comparable with LIM2
      l_piling = nn_icedyn == 1 .OR. ( nn_monocat == 2  .AND.  jpl == 1 )
      !
      IF ( ( ln_adv_Pra .AND. ln_adv_UMx ) .OR. ( .NOT.ln_adv_Pra .AND. .NOT.ln_adv_UMx ) ) THEN
         CALL ctl_stop( 'ice_adv_init: choose one and only one ice advection scheme (ln_adv_Pra or ln_adv_UMx)' )
      ENDIF
      !
   END SUBROUTINE ice_adv_init

#else
   !!----------------------------------------------------------------------
   !!   Default option         Empty Module                No sea-ice model
   !!----------------------------------------------------------------------
#endif

   !!======================================================================
END MODULE iceadv

